import { fileURLToPath } from "url";
import { PAGE_SCRIPT_ID } from "../../../vite-plugin-scripts/index.js";
import { isPage, resolveIdToUrl } from "../../util.js";
import { createRenderContext, renderPage as coreRenderPage } from "../index.js";
import { filterFoundRenderers, loadRenderer } from "../renderer.js";
import { collectMdMetadata } from "../util.js";
import { getStylesForURL } from "./css.js";
import { getScriptsForURL } from "./scripts.js";
import { createDevelopmentEnvironment } from "./environment.js";
async function loadRenderers(viteServer, settings) {
  const loader = (entry) => viteServer.ssrLoadModule(entry);
  const renderers = await Promise.all(settings.renderers.map((r) => loadRenderer(r, loader)));
  return filterFoundRenderers(renderers);
}
async function preload({
  env,
  filePath
}) {
  const renderers = await loadRenderers(env.viteServer, env.settings);
  const mod = await env.viteServer.ssrLoadModule(fileURLToPath(filePath));
  if (env.viteServer.config.mode === "development" || !(mod == null ? void 0 : mod.$$metadata)) {
    return [renderers, mod];
  }
  const modGraph = await env.viteServer.moduleGraph.getModuleByUrl(fileURLToPath(filePath));
  if (modGraph) {
    await collectMdMetadata(mod.$$metadata, modGraph, env.viteServer);
  }
  return [renderers, mod];
}
async function getScriptsAndStyles({ env, filePath }) {
  const scripts = await getScriptsForURL(filePath, env.viteServer);
  if (isPage(filePath, env.settings) && env.mode === "development") {
    scripts.add({
      props: { type: "module", src: "/@vite/client" },
      children: ""
    });
    scripts.add({
      props: {
        type: "module",
        src: await resolveIdToUrl(env.viteServer, "astro/runtime/client/hmr.js")
      },
      children: ""
    });
  }
  for (const script of env.settings.scripts) {
    if (script.stage === "head-inline") {
      scripts.add({
        props: {},
        children: script.content
      });
    } else if (script.stage === "page" && isPage(filePath, env.settings)) {
      scripts.add({
        props: { type: "module", src: `/@id/${PAGE_SCRIPT_ID}` },
        children: ""
      });
    }
  }
  const { urls: styleUrls, stylesMap } = await getStylesForURL(filePath, env.viteServer, env.mode);
  let links = /* @__PURE__ */ new Set();
  [...styleUrls].forEach((href) => {
    links.add({
      props: {
        rel: "stylesheet",
        href
      },
      children: ""
    });
  });
  let styles = /* @__PURE__ */ new Set();
  [...stylesMap].forEach(([url, content]) => {
    scripts.add({
      props: {
        type: "module",
        src: url
      },
      children: ""
    });
    styles.add({
      props: {},
      children: content
    });
  });
  return { scripts, styles, links };
}
async function renderPage(options) {
  const [renderers, mod] = options.preload;
  options.env.renderers = renderers;
  const { scripts, links, styles } = await getScriptsAndStyles({
    env: options.env,
    filePath: options.filePath
  });
  const ctx = createRenderContext({
    request: options.request,
    origin: options.origin,
    pathname: options.pathname,
    scripts,
    links,
    styles,
    route: options.route
  });
  return await coreRenderPage(mod, ctx, options.env);
}
export {
  createDevelopmentEnvironment,
  loadRenderers,
  preload,
  renderPage
};
